@@ -13,17 +13,17 @@
 %                     Write Matte Channel To MIFF File.                       %
 %                                                                             %
 %                              Software Design                                %
-%                                John Cristy                                  %
+%                                   Cristy                                    %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -40,6 +40,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/cache.h"
@@ -52,6 +53,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/monitor-private.h"
+#include "magick/pixel-accessor.h"
 #include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -83,10 +85,10 @@ static MagickBooleanType
 %
 %  The format of the RegisterMATTEImage method is:
 %
-%      unsigned long RegisterMATTEImage(void)
+%      size_t RegisterMATTEImage(void)
 %
 */
-ModuleExport unsigned long RegisterMATTEImage(void)
+ModuleExport size_t RegisterMATTEImage(void)
 {
   MagickInfo
     *entry;
@@ -95,7 +97,7 @@ ModuleExport unsigned long RegisterMATTEImage(void)
   entry->encoder=(EncodeImageHandler *) WriteMATTEImage;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("MATTE format");
-  entry->module=ConstantString("MATTE");
+  entry->magick_module=ConstantString("MATTE");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -159,8 +161,8 @@ static MagickBooleanType WriteMATTEImage(const ImageInfo *image_info,
   Image
     *matte_image;
 
-  long
-    y;
+  ImageInfo
+    *write_info;
 
   MagickBooleanType
     status;
@@ -168,16 +170,18 @@ static MagickBooleanType WriteMATTEImage(const ImageInfo *image_info,
   register const PixelPacket
     *p;
 
-  register long
+  register ssize_t
     x;
 
   register PixelPacket
     *q;
 
+  ssize_t
+    y;
+
   if (image->matte == MagickFalse)
     ThrowWriterException(CoderError,"ImageDoesNotHaveAAlphaChannel");
-  matte_image=CloneImage(image,image->columns,image->rows,MagickTrue,
-    &image->exception);
+  matte_image=CloneImage(image,0,0,MagickTrue,&image->exception);
   if (matte_image == (Image *) NULL)
     return(MagickFalse);
   (void) SetImageType(matte_image,TrueColorMatteType);
@@ -186,30 +190,35 @@ static MagickBooleanType WriteMATTEImage(const ImageInfo *image_info,
     Convert image to matte pixels.
   */
   exception=(&image->exception);
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
     p=GetVirtualPixels(image,0,y,image->columns,1,exception);
     q=QueueAuthenticPixels(matte_image,0,y,matte_image->columns,1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
-      q->red=p->opacity;
-      q->green=p->opacity;
-      q->blue=p->opacity;
-      q->opacity=OpaqueOpacity;
+      SetPixelRed(q,GetPixelOpacity(p));
+      SetPixelGreen(q,GetPixelOpacity(p));
+      SetPixelBlue(q,GetPixelOpacity(p));
+      SetPixelOpacity(q,OpaqueOpacity);
       p++;
       q++;
     }
     if (SyncAuthenticPixels(matte_image,exception) == MagickFalse)
       break;
-    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+      image->rows);
     if (status == MagickFalse)
       break;
   }
-  (void) FormatMagickString(matte_image->filename,MaxTextExtent,
-    "MIFF:%s",image->filename);
-  status=WriteImage(image_info,matte_image);
+  write_info=CloneImageInfo(image_info);
+  if ((*write_info->magick == '\0') ||
+      (LocaleCompare(write_info->magick,"MATTE") == 0))
+    (void) FormatLocaleString(matte_image->filename,MaxTextExtent,
+      "MIFF:%s",image->filename);
+  status=WriteImage(write_info,matte_image);
+  write_info=DestroyImageInfo(write_info);
   matte_image=DestroyImage(matte_image);
   return(status);
 }
